<?php 
namespace Concrete\Package\JbMegamenu\Block\JbMegamenu;

use Config;
use Core;
use Loader;
use Page;
use Permissions;
use \Concrete\Core\Page\Stack\Stack;
use StackList;
use UserInfo;
use View;
use \Concrete\Core\Block\BlockController;
use \Concrete\Core\Block\BlockType\BlockType;
use \Concrete\Core\Block\View\BlockView;
use \Concrete\Core\Asset\AssetList;

/**
 * The controller for the autonav block, which makes navigation lists and menus from C5 pages.
 */
 
class Controller extends BlockController {

	protected $btTable = 'btJbMegamenu';
	protected $btInterfaceWidth = "800";
	protected $btInterfaceHeight = "350";
	protected $btCacheBlockRecord = true;
	protected $btCacheBlockOutput = true;
	protected $btCacheBlockOutputOnPost = true;
	protected $btCacheBlockOutputForRegisteredUsers = false;
	protected $btCacheBlockOutputLifetime = 300;
	protected $btWrapperClass = 'ccm-ui';
	protected $btDefaultSet = 'navigation';
	protected $btIgnorePageThemeGridFrameworkContainer = false;
	
	protected $btExportPageColumns = array('displayPagesCID');
	
	public function getBlockTypeDescription() {
		return t("Multi-level navigation menu.");
	}
	
	public function getBlockTypeName() {
		return t("Mega Menu");
	}
	
	public $skin_type;

	public $navArray = array();
	public $cParentIDArray = array();

	public $displayPages, $displayPagesCID, $displayPagesIncludeSelf, $displaySubPages, $displaySubPageLevels, $displaySubPageLevelsNum, $orderBy, $displayUnavailablePages;
	public $haveRetrievedSelf = false;
	public $haveRetrievedSelfPlus1 = false;
	public $displaySystemPages = false;

	// private variable $displayUnapproved, used by the dashboard
	public $displayUnapproved = false;

	// haveRetrievedSelf is a variable that stores whether or not a particular tree walking operation has retrieved the current page. We use this
	// with subpage modes like enough and enough_plus1

	// displayUnavailablePages allows us to decide whether this autonav block filters out unavailable pages (pages we can't see, are restricted, etc...)
	// or whether they display them, but then restrict them when the page is actually visited
	// TODO - Implement displayUnavailablePages in the btNavigation table, and in the frontend of the autonav block

	function __construct($obj = null) {
		if (is_object($obj)) {
			switch(strtolower(get_class($obj))) {
			   case "blocktype":
				  // instantiating autonav on a particular collection page, instead of adding
				  // it through the block interface
				  $this->bID = null;
				  $c = Page::getCurrentPage();
					if (is_object($c)) {
						$this->cID = $c->getCollectionID();
						$this->cParentID = $c->getCollectionParentID();
					}
				  break;
			   case "block": // block
				  // standard block object
				  $this->bID = $obj->bID;
					$cobj = $obj->getBlockCollectionObject();
					$this->cID = ($cobj->getCollectionPointerID()) ? $cobj->getCollectionPointerOriginalID() : $cobj->getCollectionID();
					$this->displayCID = $cobj->getCollectionID();
					$this->cParentID = $cobj->cParentID;
				  break;
			}
		}

		$c = Page::getCurrentPage();
		if (is_object($c)) {
			if ($c->getCollectionPointerOriginalID() > 0) {
				$this->cID = $c->getCollectionPointerOriginalID();
			} else {
				$this->cID = $c->getCollectionID();
			}
			$this->cParentID = $c->getCollectionParentID();
		}
		
		parent::__construct($obj);
	}

	public function save($args) {
		//print_r($args);die;
		$args['displayPagesIncludeSelf'] = isset($args['displayPagesIncludeSelf']) ? 1 : 0;
		$args['displayPagesCID'] = isset($args['displayPagesCID']) ? $args['displayPagesCID'] : 0;
		$args['displaySubPageLevelsNum'] = $args['displaySubPageLevelsNum'] > 0 ? $args['displaySubPageLevelsNum'] : 0;
		$args['displayUnavailablePages'] = isset($args['displayUnavailablePages']) ? 1 : 0;
		
		if ( $args['skin_type'] == 'flat' ) {
			$args['skin'] = 'blank';
		}
		
		$args['skin'] = isset($args['skin']) ? $args['skin'] : 'custom';
		$args['effect'] = isset($args['effect']) ? $args['effect'] : 'fade';
		$args['mouseevent'] = isset($args['mouseevent']) ? $args['mouseevent'] : 'hover';
		$args['speed'] = isset($args['speed']) ? $args['speed'] : 'fast';
		$args['add_search'] = isset($args['add_search']) ? $args['add_search'] : 0;
		$args['add_login'] = isset($args['add_login']) ? $args['add_login'] : 0;
		$args['keyboardnav'] = isset($args['keyboardnav']) ? $args['keyboardnav'] : 0;
		$args['full_width'] = isset($args['full_width']) ? $args['full_width'] : 0;
		$args['custom_css'] = (isset($args['custom_css']) && $args['skin_type']=='custom') ? $args['custom_css'] : '';
		$args['responsivesupport'] = isset($args['responsivesupport']) ? $args['responsivesupport'] : 0;
		
		// 1.2
		$args['skin_type'] = isset($args['skin_type']) ? $args['skin_type'] : 'gradient';
		$args['nav_bg_color_from'] = isset($args['nav_bg_color_from']) ? $args['nav_bg_color_from'] : '#db1c1c';
		$args['nav_bg_color_to'] = isset($args['nav_bg_color_to']) ? $args['nav_bg_color_to'] : '#4d0202';
		$args['menu_bg_hover_color_from'] = isset($args['menu_bg_hover_color_from']) ? $args['menu_bg_hover_color_from'] : '#735656';
		$args['menu_bg_hover_color_to'] = isset($args['menu_bg_hover_color_to']) ? $args['menu_bg_hover_color_to'] : '#290808';
		$args['menu_font_color'] = isset($args['menu_font_color']) ? $args['menu_font_color'] : '#f5f5f5';
		$args['menu_font_hover_color'] = isset($args['menu_font_hover_color']) ? $args['menu_font_hover_color'] : '#ffffff';
		$args['menu_border_color'] = isset($args['menu_border_color']) ? $args['menu_border_color'] : '#949494';
		$args['menu_active_bg_color_from'] = isset($args['menu_active_bg_color_from']) ? $args['menu_active_bg_color_from'] : '#735656';
		$args['menu_active_bg_color_to'] = isset($args['menu_active_bg_color_to']) ? $args['menu_active_bg_color_to'] : '#290808';
		$args['menu_active_font_color'] = isset($args['menu_active_font_color']) ? $args['menu_active_font_color'] : '#ffffff';
		// submenu styles
		$args['submenu_font_color'] = isset($args['submenu_font_color']) ? $args['submenu_font_color'] : '#949494';
		$args['submenu_font_hover_color'] = isset($args['submenu_font_hover_color']) ? $args['submenu_font_hover_color'] : '#6B6B6B';
		$args['submenu_bg_hover_color_from'] = isset($args['submenu_bg_hover_color_from']) ? $args['submenu_bg_hover_color_from'] : '#6B6B6B';
		$args['submenu_bg_hover_color_to'] = isset($args['submenu_bg_hover_color_to']) ? $args['submenu_bg_hover_color_to'] : '#949494';
		$args['submenu_bg_color'] = isset($args['submenu_bg_color']) ? $args['submenu_bg_color'] : '#ffffff';
		$args['submenu_bg_color_to'] = isset($args['submenu_bg_color_to']) ? $args['submenu_bg_color_to'] : '#ffffff';
		$args['submenu_hdr_font_color'] = isset($args['submenu_hdr_font_color']) ? $args['submenu_hdr_font_color'] : '#ffffff';
		$args['submenu_hdr_font_hover_color'] = isset($args['submenu_hdr_font_hover_color']) ? $args['submenu_hdr_font_hover_color'] : '#000000';
		$args['submenu_hdr_bg_color'] = isset($args['submenu_hdr_bg_color']) ? $args['submenu_hdr_bg_color'] : '#6B6B6B';
		$args['submenu_hdr_bg_hover_color'] = isset($args['submenu_hdr_bg_hover_color']) ? $args['submenu_hdr_bg_hover_color'] : '#cccccc';
		$args['submenu_shadow'] = isset($args['submenu_shadow']) ? $args['submenu_shadow'] : 0;
		$args['submenu_rounded_corner'] = isset($args['submenu_rounded_corner']) ? $args['submenu_rounded_corner'] : 0;
		$args['navigate_on_collapse'] = isset($args['navigate_on_collapse']) ? 1 : 0;
		
		$args['enableHamburgerSupport'] = isset($args['enableHamburgerSupport']) ? 1 : 0;
		$args['disabletransparency'] = isset($args['disabletransparency']) ? 1 : 0;
		$args['breakpointSize'] = isset($args['breakpointSize']) ? $args['breakpointSize'] : 767;
		$args['hamburgerSide'] = isset($args['hamburgerSide']) ? $args['hamburgerSide'] : 'left';
		$args['hamburgerSize'] = isset($args['hamburgerSize']) ? $args['hamburgerSize'] : 'medium';
		$args['hamburger_color'] = isset($args['hamburger_color']) ? $args['hamburger_color'] : '#000';
		$args['hamburger_override_background_color'] = isset($args['hamburger_override_background_color']) ? $args['hamburger_override_background_color'] : '#000';
		$args['hamburger_override_font_color'] = isset($args['hamburger_override_font_color']) ? $args['hamburger_override_font_color'] : '#fff';
		$args['hamburger_override_menu_hover_color'] = isset($args['hamburger_override_menu_hover_color']) ? $args['hamburger_override_menu_hover_color'] : '#999';
		$args['hamburgerIconAnimationfID'] = isset($args['hamburgerIconAnimationfID']) ? $args['hamburgerIconAnimationfID'] : 0;
		$args['hamburgerIconAnimation'] = isset($args['hamburgerIconAnimation']) ? $args['hamburgerIconAnimation'] : 'morph';
		$args['hamburgerPosition'] = isset($args['hamburgerPosition']) ? $args['hamburgerPosition'] : 'relative';
			
			if ( $args['hamburgerIconAnimation'] =='custom' && $args['hamburgerIconAnimationfID']<=0) {
					throw new Exception(t('Custom Hamburger Image required'));
				
			}
			if( $args['hamburgerIconAnimation'] !='custom' ){
				$args['hamburgerIconAnimationfID']=0;
			}

		// Fonts
		if ( intval($args['font_type']) === 1 
			|| intval($args['font_type']) === 2 ) {
			if ( trim($args['font_family']) == '' ) {
				throw new Exception(t('Font Family is required when using Google Web Fonts'));
			}
		}

		$args['font_family'] = trim($args['font_family']);
		
		// Organizes the data into a 2D array for easier manipulation
		$area_set = array();
		$area_set = $this->preg_grep_keys('/^(areaParentMenu_|areaContainer_|areaPosition_)+[0-9]+$/',$this->post());
		$args['area_set'] = serialize($area_set);

		parent::save($args);
	}

	private function preg_grep_keys( $pattern, $input, $flags = 0 ) {
		$keys = preg_grep( $pattern, array_keys( $input ), $flags );
		$vals = array();
		$i = 0;
		foreach ( $keys as $key )
		{
			if ( !empty($vals[$i]) ) {
				$temp_arr = array();
				$temp_arr = preg_grep('/^areaContainer/', array_keys($vals[$i]), $flags);
				if ( empty($temp_arr) ) {
					$vals[$i]['areaContainer'] = $input[$key];
					continue;
				}
				$temp_arr = preg_grep('/^areaPosition/', array_keys($vals[$i]), $flags);
				if ( empty($temp_arr) ) {
					$vals[$i]['areaPosition'] = $input[$key];
					$i++;
				}
				
			} else {
				$vals[$i]['areaParentMenu'] = $input[$key];
			}
		}
		return $vals;
	}

	public function edit() {
		$this->set('font_type',$this->font_type);
		$this->set('font_family',$this->font_family);
	}

	public function add() {
		$this->set('font_type',$this->font_type);
		$this->set('font_family',$this->font_family);
	}

	public function view() {
		$this->set('font_type',$this->font_type);
		$this->set('font_family',$this->font_family);
	}
	
	public function registerViewAssets($outputContent = '')
    {
        $this->requireAsset('javascript', 'jquery'); //5.7+ automatically include jQuery if it's not added
    }

	public function on_start()
	{
		$al = \Concrete\Core\Asset\AssetList::getInstance();
		$al->register(
			'javascript', 'modernizr', 'blocks/jb_megamenu/js/modernizr.js'
		);

		$al->register(
			'javascript', 'keytips', 'blocks/jb_megamenu/js/jquery.KeyTips.js'
		);

		$al->register(
			'javascript', 'hoverIntent', 'blocks/jb_megamenu/js/jquery.hoverIntent.minified.js'
		);

		$al->register(
			'javascript', 'dcmegamenu', 'blocks/jb_megamenu/js/jquery.dcmegamenu.1.3.3.js'
		);

		$al->register(
			'css', 'dcmegamenu', 'blocks/jb_megamenu/css/dcmegamenu.css'
		);
	}


	public function on_page_view() {
		$bv = new BlockView($this->getBlockObject());
		$html = Core::make('helper/html');
		$uh = Loader::helper('concrete/urls');
		$bt = BlockType::getByHandle('jb_megamenu');

		if ( $this->skin_type == 'flat' ) {
			//$this->addHeaderItem($html->css('css/skins/blank.css'));
			$this->addHeaderItem($html->css($bv->getBlockURL() . '/css/skins/blank.css'));
		} else {
			$this->addHeaderItem($html->css($bv->getBlockURL() . '/css/skins/'.$this->skin.'.css'));
		}
		
	$addiinalCss='';
			if ( $this->enableHamburgerSupport) {
				//$this->addFooterItem($html->javascript($bv->getBlockURL() . '/js_manual/slideout.js'));
				$this->addHeaderItem($html->javascript($bv->getBlockURL() . '/js_manual/slideout.js'));
				$bps=$this->breakpointSize+1;
				if($this->breakpointSize!=9999){
				$addiinalCss.='
				@media only screen and (max-width: '.$this->breakpointSize.'px) {
					#mmhamburger .jbmm { display:block !important;}
					.jbmm { display:none;}
					#mmhamburger .jbmm.responsive ul.mega-menuh  { width:100%;}
				}
				@media only screen and (min-width: '.$bps.'px) {
					.mmtoggle-button,#mmhamburger .jbmm { display:none !important;}
				}
				#nav-icon1,#nav-icon2,#nav-icon3,#nav-icon4,{ z-index: 99;}';
				}else{
					$addiinalCss.='
					#mmhamburger .jbmm { display:block !important;}
					.jbmm { display:none;}
					#mmhamburger .jbmm.responsive ul.mega-menuh  { width:100%;}
					#nav-icon1,#nav-icon2,#nav-icon3,#nav-icon4{ z-index: 99;}';
				}
				
				if ( $this->hamburgerPosition) {
				//$c = Page::getCurrentPage();
				$p = new Permissions(Page::getCurrentPage());

				if ($p->canWrite()){
				$hampos = "relative";
				//$hampositionpadding=' top:50px !important;';
				}
				else{
				$hampos = $this->hamburgerPosition;
				}
										$addiinalCss.='#nav-icon1,#nav-icon2,#nav-icon3,#nav-icon4{position:'.$hampos.';'.$hampositionpadding.'}';
					
				}
				if ( $this->hamburgerSide=='right') {
					$addiinalCss.='
						.mmtoggle-button{ float:right;right:12px;top:10px;}
						#mmhamburger{ left:auto;}
					';
				}else{
					$addiinalCss.='
						.mmtoggle-button{top:10px;left:12px;}
						
					';
				}
				if($this->hamburger_color!=''){
					$addiinalCss.='
						#mmpanel #nav-icon1 span,#mmpanel #nav-icon2 span,#mmpanel #nav-icon3 span,#mmpanel #nav-icon4 span{ background-color:'.$this->hamburger_color.';}
					';
				}
				if($this->hamburger_override_background_color!=''){
					$addiinalCss.='
						div#mmhamburger { background-color:'.$this->hamburger_override_background_color.' !important; }
					';
				}
				if($this->hamburger_override_font_color!=''){
					$addiinalCss.='
						div#mmhamburger *,#mmhamburger .mega-menuh li a { color:'.$this->hamburger_override_font_color.' !important; }
					';
				}
				if($this->hamburger_override_menu_hover_color!=''){
					$addiinalCss.='
						#mmhamburger .mega-menuh li a:hover { background-color:'.$this->hamburger_override_menu_hover_color.' !important; }
					';
				}
				
			
			}
			if ( $this->hamburgerIconAnimation=='slide_left') {
				$this->addHeaderItem($html->css($bv->getBlockURL() . '/css_manual/icon_animation/slide_left.css'));
			}elseif ( $this->hamburgerIconAnimation=='slide_in') {
				$this->addHeaderItem($html->css($bv->getBlockURL() . '/css_manual/icon_animation/slide_in.css'));
			}elseif ( $this->hamburgerIconAnimation=='morph') {
				$this->addHeaderItem($html->css($bv->getBlockURL() . '/css_manual/icon_animation/morph.css'));
			}
			 
			$this->addHeaderItem('<style type="text/css">
				'.$addiinalCss.'
				</style>');
			
			
			if ( $this->hamburgerIconAnimation=='slide_left'|| $this->hamburgerIconAnimation=='slide_in' || $this->hamburgerIconAnimation=='morph') {
				$this->addFooterItem('<script type="text/javascript">$(window).load(function(){
										$("#nav-icon1,#nav-icon2,#nav-icon3,#nav-icon4" ).on( "click", function() {
										$(this).toggleClass(\'mmopen\');
										});	
									});</script>');
			}
				
	}
	
	public function buildCSS() {
		$bv = new BlockView($this->getBlockObject());
		$pieURL = 'behavior: url('.BASE_URL.Loader::helper('concrete/urls')->getToolsURL('PIE', 'jb_megamenu').');';
		$submenu_bg_color = trim($this->submenu_bg_color);
		$submenu_bg_color = !empty($submenu_bg_color) ? $submenu_bg_color : '#ffffff';
		$submenu_bg_color_to = $this->submenu_bg_color_to;
		
		// Detailed styles
		$nav_bg_from 			= $this->nav_bg_color_from;
		$nav_bg_to 				= $this->nav_bg_color_to;
		// $menu_bg_color 		= '#ccc';
		$menu_bg_hover_from 	= $this->menu_bg_hover_color_from;
		$menu_bg_hover_to 		= $this->menu_bg_hover_color_to;
		$menu_font_color 		= $this->menu_font_color;
		$menu_font_hover_color 	= $this->menu_font_hover_color;
		$menu_border_color 		= $this->menu_border_color;
		
		if ($this->skin_type!="custom"){
			$content  = '<style type="text/css">';
			$content .= 'div.sub-container.mega,div.sub-container.non-mega {';
			// Round Corners
			if ( $this->submenu_rounded_corner ) {
				$content .= '-webkit-border-radius: 0 0 8px 8px;';
				$content .= '-moz-border-radius: 0 0 8px 8px;';
				$content .= 'border-radius: 0 0 8px 8px;';
			}
			// Shadows
			if ( $this->submenu_shadow ) {
				$content .= '-webkit-box-shadow: #888 0px 2px 3px;';
				$content .= '-moz-box-shadow: #888 0px 2px 3px;';
				$content .= 'box-shadow: #888 0px 2px 3px;';
			}
			
			$content .= 'background: '. $submenu_bg_color .';';
			$content .= "background: -webkit-gradient(linear, 0 0, 0 bottom, from({$submenu_bg_color}), to({$submenu_bg_color_to}));";
			$content .= "background: -webkit-linear-gradient({$submenu_bg_color}, {$submenu_bg_color_to});";
			$content .= "background: -moz-linear-gradient({$submenu_bg_color}, {$submenu_bg_color_to});";
			$content .= "background: -ms-linear-gradient({$submenu_bg_color}, {$submenu_bg_color_to});";
			$content .= "background: -o-linear-gradient({$submenu_bg_color}, {$submenu_bg_color_to});";
			$content .= "background: linear-gradient({$submenu_bg_color}, {$submenu_bg_color_to});";
			$content .= "-pie-background: linear-gradient({$submenu_bg_color}, {$submenu_bg_color_to});";
			$content .= $pieURL;
			$content .= '}';
			
			/* Sub Menu Font color */
			$content .= 'ul.mega-menuh>li>div>ul.sub>li>a,';
			$content .= 'ul.mega-menuh>li>div>ul.sub>div.row>li.mega-hdr>ul.sub_menu>li>a {';
			$content .= 'color: '.$this->submenu_font_color.' !important;';
			$content .= '}';
			// Hover
			$content .= 'ul.mega-menuh>li>div>ul.sub>li>a:hover,';
			$content .= 'ul.mega-menuh>li>div>ul.sub>div.row>li.mega-hdr>ul.sub_menu>li>a:hover {';
			$content .= 'color: '.$this->submenu_font_hover_color.' !important;';
			$content .= '}';
			
			// Sub Menu Hover Style
			$sm_bg_color_hover_from = $this->submenu_bg_hover_color_from;
			$sm_bg_color_hover_to = $this->submenu_bg_hover_color_to;
			$imagePath = BASE_URL.$bv->getBlockURL().'/css/skins/images';
			$content .= 'ul.mega-menuh li .sub-container.non-mega li a:hover,';
			$content .= 'ul.mega-menuh .sub li.mega-hdr li a:hover {';
			$content .= 'background: '. $sm_bg_color_hover_from .';';
			// $content .= "background-image: url({$imagePath}/arrow_on.png) no-repeat 7px 10px;";
			$content .= "background: -webkit-gradient(linear, 0 0, 0 bottom, from({$sm_bg_color_hover_from}), to({$sm_bg_color_hover_to}));";
			$content .= "background: -webkit-linear-gradient({$sm_bg_color_hover_from}, {$sm_bg_color_hover_to});";
			$content .= "background: -moz-linear-gradient({$sm_bg_color_hover_from}, {$sm_bg_color_hover_to});";
			$content .= "background: -ms-linear-gradient({$sm_bg_color_hover_from}, {$sm_bg_color_hover_to});";
			$content .= "background: -o-linear-gradient({$sm_bg_color_hover_from}, {$sm_bg_color_hover_to});";
			$content .= "background: linear-gradient({$sm_bg_color_hover_from}, {$sm_bg_color_hover_to});";
			$content .= "-pie-background: linear-gradient({$sm_bg_color_hover_from}, {$sm_bg_color_hover_to});";
			$content .= '}';
			
			/* Sub Menu Heading Styles */
			$content .= 'ul.mega-menuh li .sub li.mega-hdr .mega-hdr-a {';
			$content .= 'background: '.$this->submenu_hdr_bg_color.' !important;';
			$content .= 'color: '.$this->submenu_hdr_font_color.' !important;';
			$content .= '-webkit-border-radius: 5px 5px 0 0;';
			$content .= '-moz-border-radius: 5px 5px 0 0;';
			$content .= 'border-radius: 5px 5px 0 0;';
			$content .= $pieURL;
			$content .= '}';
			// Hover
			$content .= 'ul.mega-menuh li .sub li.mega-hdr .mega-hdr-a:hover {';
			$content .= 'background: '.$this->submenu_hdr_bg_hover_color.' !important;';
			$content .= 'color: '.$this->submenu_hdr_font_hover_color.' !important;';
			$content .= '}';
			
			// Flat Style START
			if ( $this->skin_type == 'flat' ) {
				/* Active Menu Styles */
				$from = $this->menu_active_bg_color_from;
				$to = $this->menu_active_bg_color_to;
				$content .= 'ul.mega-menuh>li>a.nav-selected, ';
				$content .= 'ul.mega-menuh>li>a.nav-path-selected {';
				$content .= 'color: ' . $this->menu_active_font_color . ' !important;';
				$content .= 'background-position: 0 0 !important;';
				$content .= 'background: '. $from .' !important;';
				$content .= "background: -webkit-gradient(linear, 0 0, 0 bottom, from({$from}), to({$to}));";
				$content .= "background: -webkit-linear-gradient({$from}, {$to});";
				$content .= "background: -moz-linear-gradient({$from}, {$to});";
				$content .= "background: -ms-linear-gradient({$from}, {$to});";
				$content .= "background: -o-linear-gradient({$from}, {$to});";
				$content .= "background: linear-gradient({$from}, {$to});";
				$content .= "-pie-background: linear-gradient({$from}, {$to});";
				$content .= $pieURL;
				$content .= '}';
				
				/* Top Menu Styles */
				$content .= 'ul.mega-menuh {';
				$content .= 'height: 37px !important;';
				$content .= 'background-color: '. $nav_bg_from .';';
				$content .= "background: -webkit-gradient(linear, 0 0, 0 bottom, from({$nav_bg_from}), to({$nav_bg_to}));";
				$content .= "background: -webkit-linear-gradient({$nav_bg_from}, {$nav_bg_to});";
				$content .= "background: -moz-linear-gradient({$nav_bg_from}, {$nav_bg_to});";
				$content .= "background: -ms-linear-gradient({$nav_bg_from}, {$nav_bg_to});";
				$content .= "background: -o-linear-gradient({$nav_bg_from}, {$nav_bg_to});";
				$content .= "background: linear-gradient({$nav_bg_from}, {$nav_bg_to});";
				$content .= "-pie-background: linear-gradient({$nav_bg_from}, {$nav_bg_to});";
				$content .= $pieURL;
				$content .= '}';
				// $content .= 'ul.mega-menuh li {';
				// $content .= 'background-color: ' . $menu_bg_color . ';';
				// $content .= '}';
				$content .= 'ul.mega-menuh>li>a {';
				$content .= 'color: '.$menu_font_color.' !important;';
				$content .= '}';
				$content .= 'ul.mega-menuh>li>a:hover {';
				$content .= 'color: '.$menu_font_hover_color.' !important;';
				$content .= '}';
				$content .= 'ul.mega-menuh>li>a:hover,';
				$content .= 'ul.mega-menuh>li.mega-hover>a {';
				//$content .= 'height: 13px;';
				$content .= 'background-position: 0 0 !important;';
				$content .= 'background: '. $menu_bg_hover_from .';';
				$content .= "background: -webkit-gradient(linear, 0 0, 0 bottom, from({$menu_bg_hover_from}), to({$menu_bg_hover_to}));";
				$content .= "background: -webkit-linear-gradient({$menu_bg_hover_from}, {$menu_bg_hover_to});";
				$content .= "background: -moz-linear-gradient({$menu_bg_hover_from}, {$menu_bg_hover_to});";
				$content .= "background: -ms-linear-gradient({$menu_bg_hover_from}, {$menu_bg_hover_to});";
				$content .= "background: -o-linear-gradient({$menu_bg_hover_from}, {$menu_bg_hover_to});";
				$content .= "background: linear-gradient({$menu_bg_hover_from}, {$menu_bg_hover_to});";
				$content .= "-pie-background: linear-gradient({$menu_bg_hover_from}, {$menu_bg_hover_to});";
				$content .= $pieURL;
				$content .= '}';
				$content .= 'ul.mega-menuh>li {';
				$content .= 'border-right: 1px solid '.$menu_border_color.' !important;';
				$content .= '}';
				$content .= 'ul.mega-menuh li:last-child,';
				$content .= 'ul.mega-menuh li.list-child {';
				$content .= 'border-right: none !important;';
				$content .= '}';
			}
			// Flat Style END
			$content .= '</style>';
			return $content;
		}
	}
	
	//public $accesskeyarr = array(); //setup array for assigning access keys (keyboard shortcuts)
	public function getAccessKey($ni,&$accesskeyarr){
		$coll = $ni->getCollectionObject();
		$acck = $coll->getCollectionAttributeValue('megamenu_accesskey');
		if ($acck){array_push($accesskeyarr, $acck);return strtoupper($acck);}
		else{return $this->assignAccessKeyAuto($ni->getName(),$accesskeyarr);}
	}

	public function assignAccessKeyAuto($varPageName,&$accesskeyarr){
		
		if ($varPageName){
			$wordarray = str_split($varPageName);
			foreach($wordarray as $char) {
				if(in_array($char,$accesskeyarr)==false){
					array_push($accesskeyarr, $char);
					return strtoupper($char);
				}
			}
			return $this->assignAccessKeyAuto(null,$accesskeyarr); //fallback to random letter
		}
		else{
			$int = rand(0,35);
			$a_z = "ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890";
			$rand_letter = $a_z[$int];
			if(in_array($rand_letter,$accesskeyarr)){
				return $this->assignAccessKeyAuto(null,$accesskeyarr);
			}
			else{
				array_push($accesskeyarr, $rand_letter);
				return strtoupper($rand_letter);
			}
		}
	}

	public function getBlockFromScrapbookAfter($menuelement,$srapitemname){
		return $this->getBlockFromScrapbook($menuelement,$srapitemname,"append");
	}
	
	public function getBlockFromScrapbookBefore($menuelement,$srapitemname){
		return $this->getBlockFromScrapbook($menuelement,$srapitemname,"prepend");
	}
	
	public function getBlockFromScrapbook($menuelement,$srapitemname,$placement){
		//this should be called inside a "script" tag inside a "$(document).ready(function() {"
		if ($srapitemname && $menuelement) {
			$block = Stack::getByName($srapitemname);

			// if( $block && $block->bID ) { //check scrapbook item exists
			if( (get_class($block) == 'Stack' && $block) 
				|| (get_class($block) == 'Block' && $block && $block->bID) ) {
				$printed = "";
				ob_start();
				$block->display();
				$printed = ob_get_contents();
				ob_end_clean();
				
				$printed = str_replace('"','\"',$printed);
				$printed = str_replace('\n','',$printed);
				$printed = str_replace('\r','',$printed);
				$printed = str_replace(chr(10),'',$printed);
				$printed = str_replace(chr(13),'',$printed);
				
				$cleanelementname = str_replace('"','',$menuelement);
				$cleanelementname = str_replace('\'','',$menuelement);
				$cleanelementname = str_replace('.','',$cleanelementname);
				$cleanelementname = str_replace('#','',$cleanelementname);
				$cleanelementname = str_replace(' ','',$cleanelementname);
				
				$jqoutput = '$("'.$menuelement.'").'.$placement.'("<div id=\'mmext-'.$placement.'-'.$cleanelementname.'\'>'.$printed.'</div>");'.chr(10);
				
				return $jqoutput;
			
			}
		
		}
	}

	public function getAreaSet() {
		return unserialize($this->area_set);
	}

	public function initArea($menuelement) {
		$divContainer  = '<div class="area-container top top-'.$menuelement.' mmccont sub"></div><div class="jbmmcl"></div>';
		$divContainer .= '<div class="area-container left left-'.$menuelement.' mmccont sub"></div>';
		$jqoutput  = "$('ul#mega-menuh-".$this->bID." .pid-{$menuelement}').parent().prepend('{$divContainer}');".chr(10);

		$divContainer = '<div class="area-container right right-'.$menuelement.' mmccont sub"></div><div class="jbmmcl"></div>';
		$divContainer .= '<div class="area-container bottom bottom-'.$menuelement.' mmccont sub"></div>';
		$jqoutput .= "$('ul#mega-menuh-".$this->bID." .pid-{$menuelement}').parent().append('{$divContainer}');".chr(10);

		return $jqoutput;
	}

	public function generateBlock($menuelement,$srapitemname,$position='top') {
		//this should be called inside a "script" tag inside a "$(document).ready(function() {"
		if ($srapitemname && $menuelement) {
			$block = Stack::getByName($srapitemname);

			if( (get_class($block) == 'Concrete\Core\Page\Stack\Stack' && $block)
				|| (get_class($block) == 'Block' && is_object($block)) ) {
				$printed = "";
				ob_start();
				$block->display();
				$printed = ob_get_contents();
				ob_end_clean();
				
				$printed = str_replace('"','\"',$printed);
				$printed = str_replace("'","\'",$printed);
				$printed = str_replace('\n','',$printed);
				$printed = str_replace('\r','',$printed);
				$printed = str_replace(chr(10),'',$printed);
				$printed = str_replace(chr(13),'',$printed);
				
				$cleanelementname = str_replace('"','',$menuelement);
				$cleanelementname = str_replace('\'','',$menuelement);
				$cleanelementname = str_replace('.','',$cleanelementname);
				$cleanelementname = str_replace('#','',$cleanelementname);
				$cleanelementname = str_replace(' ','',$cleanelementname);


				$jqoutput = "$('.area-container.{$position}-{$menuelement}').append('{$printed}');".chr(10);

				return $jqoutput;
			
			}
		
		}
	}
	
	public function getBlockContainerList() {
		$containerList = array();
		Loader::model('stack/list');
		$stm = new StackList();
		// $stm->filterByGlobalAreas();
		// Use filterByGlobalAreas() or filterByUserAdded()
		$stacks = $stm->get();

		foreach($stacks as $st) {
			$containerList[$st->getCollectionName()] = $st->getCollectionName();
			//  Use $st->getCollectionName(),  $st->getCollectionHandle(), etc.
		}

		return $containerList;
	}

	public function getContent() {
		/* our templates expect a variable not an object */
		$con = array();
		foreach($this as $key => $value) {
			$con[$key] = $value;
		}
		return $con;
	}
	
	public function getChildPages($c) {
		// a quickie
		$db = Loader::db();
		$r = $db->query("select cID from Pages where cParentID = ? order by cDisplayOrder asc", array($c->getCollectionID()));
		$pages = array();
		while ($row = $r->fetchRow()) {
			$pages[] = Page::getByID($row['cID'], 'ACTIVE');
		}
		return $pages;
	}
	
	/**
	 * This function is used by the getNavItems() method to generate the raw "pre-processed" nav items array.
	 * It also must exist as a separate function to preserve backwards-compatibility with older autonav templates.
	 * Warning: this function has side-effects -- if this gets called twice, items will be duplicated in the nav structure!
	 */
	function generateNav() {
		// Initialize Nav Array
		$this->navArray = array();

		if (isset($this->displayPagesCID) && !Core::make('helper/validation/numbers')->integer($this->displayPagesCID)) {
			$this->displayPagesCID = 0;
		}

		$db = Loader::db();
		// now we proceed, with information obtained either from the database, or passed manually from
		$orderBy = "";
		/*switch($this->orderBy) {
        switch($this->orderBy) {
            case 'display_asc':
                $orderBy = "order by Collections.cDisplayOrder asc";
                break;
            case 'display_desc':
                $orderBy = "order by Collections.cDisplayOrder desc";
                break;
            case 'chrono_asc':
                $orderBy = "order by cvDatePublic asc";
                break;
            case 'chrono_desc':
                $orderBy = "order by cvDatePublic desc";
                break;
            case 'alpha_desc':
                $orderBy = "order by cvName desc";
                break;
            default:
                $orderBy = "order by cvName asc";
                break;
        }*/
		switch ($this->orderBy) {
			case 'display_asc':
				$orderBy = "order by Pages.cDisplayOrder asc";
				break;
			case 'display_desc':
				$orderBy = "order by Pages.cDisplayOrder desc";
				break;
			default:
				$orderBy = '';
				break;
		}
		$level = 0;
		$cParentID = 0;
		switch ($this->displayPages) {
			case 'current':
				$cParentID = $this->cParentID;
				if ($cParentID < 1) {
					$cParentID = 1;
				}
				break;
			case 'top':
				// top level actually has ID 1 as its parent, since the home page is effectively alone at the top
				$cParentID = 1;
				break;
			case 'above':
				$cParentID = $this->getParentParentID();
				break;
			case 'below':
				$cParentID = $this->cID;
				break;
			case 'second_level':
				$cParentID = $this->getParentAtLevel(2);
				break;
			case 'third_level':
				$cParentID = $this->getParentAtLevel(3);
				break;
			case 'custom':
				$cParentID = $this->displayPagesCID;
				break;
			default:
				$cParentID = 1;
				break;
		}

		if ($cParentID != null) {

			/*

            $displayHeadPage = false;

            if ($this->displayPagesIncludeSelf) {
                $q = "select Pages.cID from Pages where Pages.cID = '{$cParentID}' and cIsTemplate = 0";
                $r = $db->query($q);
                if ($r) {
                    $row = $r->fetchRow();
                    $displayHeadPage = true;
                    if ($this->displayUnapproved) {
                        $tc1 = Page::getByID($row['cID'], "RECENT");
                    } else {
                        $tc1 = Page::getByID($row['cID'], "ACTIVE");
                    }
                    $tc1v = $tc1->getVersionObject();
                    if (!$tc1v->isApproved() && !$this->displayUnapproved) {
                        $displayHeadPage = false;
                    }
                }
            }

            if ($displayHeadPage) {
                $level++;
            }
            */

			if ($this->displaySubPages == 'relevant' || $this->displaySubPages == 'relevant_breadcrumb') {
				$this->populateParentIDArray($this->cID);
			}

			$this->getNavigationArray($cParentID, $orderBy, $level);

			// if we're at the top level we add home to the beginning
			if ($cParentID == 1) {
				if ($this->displayUnapproved) {
					$tc1 = Page::getByID(HOME_CID, "RECENT");
				} else {
					$tc1 = Page::getByID(HOME_CID, "ACTIVE");
				}
				$niRow = array();
				$niRow['cvName'] = $tc1->getCollectionName();
				$niRow['cID'] = HOME_CID;
				$niRow['cvDescription'] = $tc1->getCollectionDescription();
				$niRow['cPath'] = Loader::helper('navigation')->getLinkToCollection($tc1);

				$ni = new JbNavItem($niRow, 0);
				$ni->setCollectionObject($tc1);

				array_unshift($this->navArray, $ni);
			}

			/*

            if ($displayHeadPage) {
                $niRow = array();
                $niRow['cvName'] = $tc1->getCollectionName();
                $niRow['cID'] = $row['cID'];
                $niRow['cvDescription'] = $tc1->getCollectionDescription();
                $niRow['cPath'] = $tc1->getCollectionPath();

                $ni = new NavItem($niRow, 0);
                $level++;
                $ni->setCollectionObject($tc1);

                array_unshift($this->navArray, $ni);
            }
            */

		}

		return $this->navArray;
	}

	function getParentAtLevel($level) {
		// this function works in the following way
		// we go from the current collection up to the top level. 
		// Then we find the parent Id at the particular level specified, and begin our
		// autonav from that point

		$this->populateParentIDArray($this->cID);

		$idArray = array_reverse($this->cParentIDArray);
		$this->cParentIDArray = array();
		if ($level - count($idArray) == 0) {
			// This means that the parent ID array is one less than the item
			// we're trying to grab - so we return our CURRENT page as the item to get
			// things under
			return $this->cID;
		}
		
		if (isset($idArray[$level])) {
			return $idArray[$level];
		} else {
			return null;
		}
	}

	/**
	 * This returns a key value pair of cID and Page Title that are
	 * at the first level relative to the level set on the block settings.
	 * @return array List of cIDs => Page Title
	 */
	public function getCurrentFirstLevelMenus() {
		$firstLevelMenus = array();
		$excludeChildrenCIDs = array();
		$aBlocks = $this->generateNav();
		foreach($aBlocks as $ni) {
			$_c = $ni->getCollectionObject();
			$excludeChildren = $_c->getCollectionAttributeValue('hide_children_from_megamenu');
			if( $excludeChildren 
				|| in_array($_c->getCollectionParentID(),$excludeChildrenCIDs) ) {
				$excludeChildrenCIDs[]=intval($_c->getCollectionID()); 
			}
			
			if ( !$_c->getCollectionAttributeValue('exclude_nav') 
					&& !in_array($_c->getCollectionParentID(),$excludeChildrenCIDs) ) {
				if ( $ni->getLevel() == 0 ) {
					$firstLevelMenus[$_c->getCollectionID()] = Page::getByID($_c->getCollectionID())->getCollectionName();
				}
			}
		}
		return $firstLevelMenus;
	}
	
	protected function displayPage($tc) {
	
		if ($tc->isSystemPage() && (!$this->displaySystemPages)) {
			if ($tc->getCollectionPath() == '/members' && Config::get('ENABLE_USER_PROFILES')) {
				if ($this->displayUnavailablePages) {
					return true;
				} else {
					$tcp = new Permissions($tc);
					return $tcp->canRead();
				}
			}
			
			return false;
		}
		
		$tcv = $tc->getVersionObject();
		if ((!is_object($tcv)) || (!$tcv->isApproved() && !$this->displayUnapproved)) { 
			return false;
		}
		
		if ($this->displayUnavailablePages == false) {
			$tcp = new Permissions($tc);
			if (!$tcp->canRead() && ($tc->getCollectionPointerExternalLink() == null)) {
				return false;
			}
		}

		return true;
	}

	function getNavigationArray($cParentID, $orderBy, $currentLevel) {
		// increment all items in the nav array with a greater $currentLevel
		
		foreach($this->navArray as $ni) {
			if ($ni->getLevel() + 1 < $currentLevel) {
				$ni->hasChildren = true;
			}
		}
		
		$db = Loader::db();
		$navSort = $this->navSort;
		$sorted_array = $this->sorted_array;
		$navObjectNames = $this->navObjectNames;

		$q = "select Pages.cID from Pages where cIsTemplate = 0 and cIsActive = 1 and cParentID = '{$cParentID}' {$orderBy}";
		$r = $db->query($q);
		if ($r) {
			while ($row = $r->fetchRow()) {
				if ($this->displaySubPages != 'relevant_breadcrumb' || (in_array(
							$row['cID'],
							$this->cParentIDArray) || $row['cID'] == $this->cID)
				) {
					/*
                    if ($this->haveRetrievedSelf) {
                        // since we've already retrieved self, and we're going through again, we set plus 1
                        $this->haveRetrievedSelfPlus1 = true;
                    } else
                    */

					if ($this->haveRetrievedSelf && $cParentID == $this->cID) {
						$this->haveRetrievedSelfPlus1 = true;
					} else {
						if ($row['cID'] == $this->cID) {
							$this->haveRetrievedSelf = true;
						}
					}

					$displayPage = true;
					if ($this->displayUnapproved) {
						$tc = Page::getByID($row['cID'], "RECENT");
					} else {
						$tc = Page::getByID($row['cID'], "ACTIVE");
					}

					$displayPage = $this->displayPage($tc);

					if ($displayPage) {
						$niRow = array();
						$niRow['cvName'] = $tc->getCollectionName();
						$niRow['cID'] = $row['cID'];
						$niRow['cvDescription'] = $tc->getCollectionDescription();
						$niRow['cPath'] = Core::make('helper/navigation')->getLinkToCollection($tc);
						$niRow['cPointerExternalLink'] = $tc->getCollectionPointerExternalLink();
						$niRow['cPointerExternalLinkNewWindow'] = $tc->openCollectionPointerExternalLinkInNewWindow();
						$dateKey = strtotime($tc->getCollectionDatePublic());

						$ni = new JbNavItem($niRow, $currentLevel);
						$ni->setCollectionObject($tc);
						// $this->navArray[] = $ni;
						$navSort[$niRow['cID']] = $dateKey;
						$sorted_array[$niRow['cID']] = $ni;

						$_c = $ni->getCollectionObject();
						$object_name = $_c->getCollectionName();
						$navObjectNames[$niRow['cID']] = $object_name;
					}
				}
			}
			// end while -- sort navSort

			// Joshua's Huge Sorting Crap
			if ($navSort) {
				$sortit = 0;
				if ($this->orderBy == "chrono_asc") {
					asort($navSort);
					$sortit = 1;
				}
				if ($this->orderBy == "chrono_desc") {
					arsort($navSort);
					$sortit = 1;
				}

				if ($sortit) {
					foreach ($navSort as $sortCID => $sortdatewhocares) {
						// create sorted_array
						$this->navArray[] = $sorted_array[$sortCID];

						#############start_recursive_crap
						$retrieveMore = false;
						if ($this->displaySubPages == 'all') {
							if ($this->displaySubPageLevels == 'all' || ($this->displaySubPageLevels == 'custom' && $this->displaySubPageLevelsNum > $currentLevel)) {
								$retrieveMore = true;
							}
						} else {
							if (($this->displaySubPages == "relevant" || $this->displaySubPages == "relevant_breadcrumb") && (in_array(
										$sortCID,
										$this->cParentIDArray) || $sortCID == $this->cID)
							) {
								if ($this->displaySubPageLevels == "enough" && $this->haveRetrievedSelf == false) {
									$retrieveMore = true;
								} else {
									if ($this->displaySubPageLevels == "enough_plus1" && $this->haveRetrievedSelfPlus1 == false) {
										$retrieveMore = true;
									} else {
										if ($this->displaySubPageLevels == 'all' || ($this->displaySubPageLevels == 'custom' && $this->displaySubPageLevelsNum > $currentLevel)) {
											$retrieveMore = true;
										}
									}
								}
							}
						}
						if ($retrieveMore) {
							$this->getNavigationArray($sortCID, $orderBy, $currentLevel + 1);
						}
						#############end_recursive_crap
					}
				}

				$sortit = 0;
				if ($this->orderBy == "alpha_desc") {
					$navObjectNames = array_map('strtolower', $navObjectNames);
					arsort($navObjectNames);
					$sortit = 1;
				}

				if ($this->orderBy == "alpha_asc") {
					$navObjectNames = array_map('strtolower', $navObjectNames);
					asort($navObjectNames);
					$sortit = 1;
				}

				if ($sortit) {
					foreach ($navObjectNames as $sortCID => $sortnameaction) {
						// create sorted_array
						$this->navArray[] = $sorted_array[$sortCID];

						#############start_recursive_crap
						$retrieveMore = false;
						if ($this->displaySubPages == 'all') {
							if ($this->displaySubPageLevels == 'all' || ($this->displaySubPageLevels == 'custom' && $this->displaySubPageLevelsNum > $currentLevel)) {
								$retrieveMore = true;
							}
						} else {
							if (($this->displaySubPages == "relevant" || $this->displaySubPages == "relevant_breadcrumb") && (in_array(
										$sortCID,
										$this->cParentIDArray) || $sortCID == $this->cID)
							) {
								if ($this->displaySubPageLevels == "enough" && $this->haveRetrievedSelf == false) {
									$retrieveMore = true;
								} else {
									if ($this->displaySubPageLevels == "enough_plus1" && $this->haveRetrievedSelfPlus1 == false) {
										$retrieveMore = true;
									} else {
										if ($this->displaySubPageLevels == 'all' || ($this->displaySubPageLevels == 'custom' && $this->displaySubPageLevelsNum > $currentLevel)) {
											$retrieveMore = true;
										}
									}
								}
							}
						}
						if ($retrieveMore) {
							$this->getNavigationArray($sortCID, $orderBy, $currentLevel + 1);
						}
						#############end_recursive_crap
					}
				}

				$sortit = 0;
				if ($this->orderBy == "display_desc") {
					$sortit = 1;
				}
				if ($this->orderBy == "display_asc") {
					$sortit = 1;
				}

				if ($sortit) {
					// for display order? this stuff is already sorted...
					foreach ($navObjectNames as $sortCID => $sortnameaction) {
						// create sorted_array
						$this->navArray[] = $sorted_array[$sortCID];

						#############start_recursive_crap
						$retrieveMore = false;
						if ($this->displaySubPages == 'all') {
							if ($this->displaySubPageLevels == 'all' || ($this->displaySubPageLevels == 'custom' && $this->displaySubPageLevelsNum > $currentLevel)) {
								$retrieveMore = true;
							}
						} else {
							if (($this->displaySubPages == "relevant" || $this->displaySubPages == "relevant_breadcrumb") && (in_array(
										$sortCID,
										$this->cParentIDArray) || $sortCID == $this->cID)
							) {
								if ($this->displaySubPageLevels == "enough" && $this->haveRetrievedSelf == false) {
									$retrieveMore = true;
								} else {
									if ($this->displaySubPageLevels == "enough_plus1" && $this->haveRetrievedSelfPlus1 == false) {
										$retrieveMore = true;
									} else {
										if ($this->displaySubPageLevels == 'all' || ($this->displaySubPageLevels == 'custom' && $this->displaySubPageLevelsNum > $currentLevel)) {
											$retrieveMore = true;
										}
									}
								}
							}
						}
						if ($retrieveMore) {
							$this->getNavigationArray($sortCID, $orderBy, $currentLevel + 1);
						}
						#############end_recursive_crap
					}
				}
			}
			// End Joshua's Huge Sorting Crap

		}
	}

	function populateParentIDArray($cID) {
		// returns an array of collection IDs going from the top level to the current item
		$db = Loader::db();
		$cParentID = Page::getCollectionParentIDFromChildID($cID);
		if ($cParentID > -1) {
			if ($cParentID != $stopAt) {
				if (!in_array($cParentID, $this->cParentIDArray)) {
					$this->cParentIDArray[] = $cParentID;
				}
				$this->populateParentIDArray($cParentID);
			}
		}

	}
	
	/** 
	 * heh. probably should've gone the simpler route and named this getGrandparentID()
	 */
	function getParentParentID() {
		// this has to be the stupidest name of a function I've ever created. sigh
		$cParentID = Page::getCollectionParentIDFromChildID($this->cParentID);
		return ($cParentID) ? $cParentID : 0;
	}
	
	/**
	 * New and improved version of "generateNav()" function.
	 * Use this unless you need to maintain backwards compatibility with older custom templates.
	 *
	 * Pass in TRUE for the $ignore_exclude_nav arg if you don't want to exclude any pages
	 *  (for both the "exclude_nav" and "exclude_subpages_from_nav" attribute).
	 * This is useful for breadcrumb nav menus, for example.
	 * 
	 * Historical note: this must stay a function that gets called by the view templates
	 * (as opposed to just having the view() method set the variables)
	 * because we need to maintain the generateNav() function for backwards compatibility with
	 * older custom templates... and that function unfortunately has side-effects so it cannot
	 * be called more than once per request (otherwise there will be duplicate items in the nav menu).
	 */
	public function getNavItems($ignore_exclude_nav = false) {
		$c = Page::getCurrentPage();

		//Create an array of parent cIDs so we can determine the "nav path" of the current page
		$inspectC = $c;
		$selectedPathCIDs = array($inspectC->getCollectionID());
		$parentCIDnotZero=true;
		while ($parentCIDnotZero) {
			$cParentID = $inspectC->cParentID;
			if (!intval($cParentID)) {
				$parentCIDnotZero=false;
			} else {
				if ($cParentID != HOME_CID) {
					$selectedPathCIDs[] = $cParentID; //Don't want home page in nav-path-selected
				}
				$inspectC = Page::getById($cParentID, 'ACTIVE');
			}
		}

		//Retrieve the raw "pre-processed" list of all nav items (before any custom attributes are considered)
		$allNavItems = $this->generateNav();
		
		//Remove excluded pages from the list (do this first because some of the data prep code needs to "look ahead" in the list)
		$includedNavItems = array();
		$excluded_parent_level = 9999; //Arbitrarily high number denotes that we're NOT currently excluding a parent (because all actual page levels will be lower than this)
		$exclude_children_below_level = 9999; //Same deal as above. Note that in this case "below" means a HIGHER number (because a lower number indicates higher placement in the sitemp -- e.g. 0 is top-level)
		foreach ($allNavItems as $ni) {
			$_c = $ni->getCollectionObject();
			$current_level = $ni->getLevel();

			if ($this->excludeFromNavViaAttribute($_c) && ($current_level <= $excluded_parent_level)) {
			    $excluded_parent_level = $current_level;
			    $exclude_page = true;
			} else {
			    if (($current_level > $excluded_parent_level) || ($current_level > $exclude_children_below_level)) {
			        $exclude_page = true;
			    } else {
			        $excluded_parent_level = 9999; //Reset to arbitrarily high number to denote that we're no longer excluding a parent
			        $exclude_children_below_level = $_c->getAttribute(
			                                           'exclude_subpages_from_nav') ? $current_level : 9999;
			        $exclude_page = false;
			    }
			}

			if (!$exclude_page || $ignore_exclude_nav) {
				$includedNavItems[] = $ni;
			}
		}

		//Prep all data and put it into a clean structure so markup output is as simple as possible
		$navItems = array();
		$accesskeyarr = array();
		$navItemCount = count($includedNavItems);
		for ($i = 0; $i < $navItemCount; $i++) {
			$ni = $includedNavItems[$i];
			$_c = $ni->getCollectionObject();
			$current_level = $ni->getLevel();

			//Link target (e.g. open in new window)
			$target = $ni->getTarget();
			$target = empty($target) ? '_self' : $target;

			//Link URL
			$pageLink = false;
			if ( $_c->getAttribute('replace_link_with_parent_nav') ) {
				$page = Page::getByID($_c->getCollectionParentID());
				if ($page instanceof Page) {
					$pageLink = Core::make('helper/navigation')->getLinkToCollection($page);
				}
			} elseif ( $_c->getAttribute('replace_link_with_first_in_nav') ) {
				$subPage = $_c->getFirstChild();
				if ($subPage instanceof Page) {
					$pageLink = Core::make('helper/navigation')->getLinkToCollection($subPage);
				}
			}

			if (!$pageLink) {
				$pageLink = $ni->getURL();
			}

			//Current/ancestor page
			$selected = false;
			$path_selected = false;
			if ($c->getCollectionID() == $_c->getCollectionID()) {
				$selected = true; //Current item is the page being viewed
				$path_selected = true;
			} elseif (in_array($_c->getCollectionID(), $selectedPathCIDs)) {
				$path_selected = true; //Current item is an ancestor of the page being viewed
			}

			//Calculate difference between this item's level and next item's level so we know how many closing tags to output in the markup
			$next_level = isset($includedNavItems[$i+1]) ? $includedNavItems[$i+1]->getLevel() : 0;
			$levels_between_this_and_next = $current_level - $next_level;

			//Determine if this item has children (can't rely on $ni->hasChildren() because it doesn't ignore excluded items!)
			$has_children = $next_level > $current_level;

			//Calculate if this is the first item in its level (useful for CSS classes)
			$prev_level = isset($includedNavItems[$i-1]) ? $includedNavItems[$i-1]->getLevel() : -1;
			$is_first_in_level = $current_level > $prev_level;

			//Calculate if this is the last item in its level (useful for CSS classes)
			$is_last_in_level = true;
			for ($j = $i+1; $j < $navItemCount; $j++) {
				if ($includedNavItems[$j]->getLevel() == $current_level) {
					//we found a subsequent item at this level (before this level "ended"), so this is NOT the last in its level
					$is_last_in_level = false;
					break;
				}
				if ($includedNavItems[$j]->getLevel() < $current_level) {
					//we found a previous level before any other items in this level, so this IS the last in its level
					$is_last_in_level = true;
					break;
				}
			} //If loop ends before one of the "if" conditions is hit, then this is the last in its level (and $is_last_in_level stays true)

			//Custom CSS class
			$attribute_class = $_c->getAttribute('nav_item_class');
			$attribute_class = empty($attribute_class) ? '' : $attribute_class;

			//Page ID stuff
			$item_cid = $_c->getCollectionID();
			$is_home_page = ($item_cid == HOME_CID);

			// Omit item's anchor tag if the page is not linked.
			$isPageLinked = true;
			if ( $_c->getCollectionAttributeValue('dont_link') && $current_level == 0 ) {
				//top level menu items that are not linked, still have # link
				$isPageLinked = true;
				$pageLink = "#"; 
			} elseif ( $_c->getCollectionAttributeValue('dont_link') && $current_level != 0 ) {
				$isPageLinked = false;
			} else {
				$isPageLinked = true;
			}

			// Change link according to Page Attribute Force SSL
			if ($_c->getCollectionAttributeValue('force_ssl')) {
				$pageLink = str_replace("http","https",BASE_URL . $pageLink);
			}
			if ($_c->getCollectionAttributeValue('force_no_ssl')) {
				$pageLink = str_replace("https","http",BASE_URL . $pageLink);
			}

			$accesskey = '';
			if ( $this->keyboardnav ) {
				if ( is_object($ni) ) {
					$accesskey = $this->getAccessKey($ni,$accesskeyarr);
				}
			}

			//Package up all the data
			$navItem = new \stdClass();
			$navItem->url = $pageLink;
			$navItem->accesskey = $accesskey;
			$navItem->isPageLinked = $isPageLinked;

			$translate = $this->get('translate');
			$name = (isset($translate) && $translate == true) ? t($ni->getName()) : $ni->getName();
			$text = Core::make('helper/text');
			$navItem->name = $text->entities($name);

			$navItem->isExternalLink = $ni->getCollectionObject()->isExternalLink();
			$navItem->target = $target;
			$navItem->level = $current_level + 1; //make this 1-based instead of 0-based (more human-friendly)
			$navItem->subDepth = $levels_between_this_and_next;
			$navItem->hasSubmenu = $has_children;
			$navItem->isFirst = $is_first_in_level;
			$navItem->isLast = $is_last_in_level;
			$navItem->isCurrent = $selected;
			$navItem->inPath = $path_selected;
			$navItem->attrClass = $attribute_class;
			$navItem->isHome = $is_home_page;
			$navItem->cID = $item_cid;
			$navItem->cObj = $_c;

			$navItems[] = $navItem;
		}
		
		return $navItems;
	}

	public function excludeFromNavViaAttribute($c)
    {
        return $c->getAttribute('exclude_nav');
    }

}
